/**
 * plugin.js
 */

'use strict';

import CONST    from './constant'


export default class Plugin extends React.Component {

  constructor(props) {
    super(props);
    this.onItemClick = this.onItemClick.bind(this);
  }

  /**
   * 获取有效的插件
   * - 插件如果指定了租户，那么租户需要匹配
   * - 如果参数type指定了，那么只返回type匹配的插件
   * @param type    组件类型
   * @param subType 组件子类型 (type === form 时有效)
   * @returns {Array}
   */
  static list(type, subType) {
    if (typeof plugin === 'undefined') {
      return [];
    }

    // Current tenant
    const tenant = light.util.code();

    return plugin.filter(item => {
      if (!item.defaultProps) {
        console.log('The plugin default props is not defined.');
        return false;
      }

      if (!item.defaultProps.id) {
        console.log('The plugin id is not defined.');
        return false;
      }

      const scope   = item.defaultProps.scope
        , code      = item.defaultProps.code
        , codeEx    = item.defaultProps.codeExclude
        , scopeType = item.defaultProps.scopeType;

      if (!scope) {
        console.log('The plugin scope is not defined.');
        return false;
      }

      // 如果指定了除外的租户，那么租户需要匹配
      if (codeEx && tenant) {
        if ((Array.isArray(codeEx) ? codeEx : [codeEx]).includes(tenant)) {
          return false;
        }
      }

      // 如果指定了租户，那么租户需要匹配
      if (code && tenant) {
        if ((Array.isArray(code) ? code : [code]).indexOf(tenant) < 0) {
          return false;
        }
      }

      // type=all时，可以获取所有插件的一览
      if (type === 'all') {
        return true;
      }

      // 如果指定了表单的子类型，那么需要判断表单类型与当前的表单类型匹配
      if (scopeType && subType) {
        if (!(Array.isArray(scopeType) ? scopeType : [scopeType]).includes(subType)) {
          return false;
        }
      }

      return (Array.isArray(scope) ? scope : [scope]).indexOf(type) >= 0;
    });
  }

  render() {

    const items = Plugin.list(this.props.type, this.props.data.type);
    if (items.length <= 0) {
      return React.DOM.div({style: {padding: '25px'}}, 'NO DATE');
    }

    // 名称排序
    items.sort((a, b) => {
      return a.defaultProps.id.localeCompare(b.defaultProps.id);
    });

    return React.DOM.div({className: 'list-group plugin-panel'},
      items.map(item => {

        const extend = this.props.data.extend || {}
          , id = item.defaultProps.id
          , state = extend.hasOwnProperty(id) ? 'on' : 'off';

        return React.DOM.a({
            'data-id': id,
            'data-state': state,
            key: id,
            className: 'list-group-item',
            onClick: this.onItemClick
          },
          React.DOM.i({
            className: `fa fa-toggle-${state}`,
            style: {float: 'right', fontSize: '24px', marginTop: '-2px'}
          }),
          React.DOM.h4({className: 'list-group-item-heading'}, item.defaultProps.id),
          React.DOM.p({className: 'list-group-item-text'}, item.defaultProps.description)
        )
      })
    );
  }

  onItemClick(event) {

    const id = event.currentTarget.getAttribute('data-id')
      , state = event.currentTarget.getAttribute('data-state');

    let extend = this.props.data.extend || {};
    if (state === 'off') {
      if (!extend.hasOwnProperty(id)) {
        extend[id] = null;
      }
    } else {
      delete extend[id];
    }

    this.props.emit(CONST.EVENT.CHANGE, {name: 'extend', value: extend});
  }
}